'use strict';

require('core-js/fn/string/includes');

var config = require('../../config.json');

var DEFAULT_COMMENT_TOP_PADDING = -1;
var ALLOWED_COMMENT_TYPE = 'CommentBlock';
var ALLOWED_DIRECTIVES = config.doctrineConfig.tags.filter(function (tag) {
    return tag !== 'name';
}).map(function (tag) {
    return '@' + tag;
});

/**
 * @param {Object} options
 * @returns {String|Boolean}
 */
function getDirective(options) {
    var directive = void 0;

    if ('useDirective' in options && options.useDirective !== true) {
        directive = options.useDirective;
    } else {
        directive = config.default.useDirective;
    }

    return directive;
}

/**
 * @param {NodePath} path
 * @param {PluginPass} state
 * @param {Boolean} hasGlobalDirective
 * @returns {String|null}
 */
module.exports = function (path, state, hasGlobalDirective) {
    var node = path.node;

    if (!node.leadingComments) {
        return null;
    }

    var comments = node.leadingComments;

    var previousNode = path.key !== 0 ? path.getSibling(path.key - 1) : null;
    var previousNodeEnd = void 0;

    if (previousNode && previousNode.node) {
        previousNodeEnd = previousNode.node.end || DEFAULT_COMMENT_TOP_PADDING;
    } else {
        previousNodeEnd = DEFAULT_COMMENT_TOP_PADDING;
    }

    var functionDeclarationStart = void 0;

    /**
     * Fix for annoying edge case, when ObjectProperty node doesn't have start index
     * Workaround: find function body and get it's start index.
     */
    if (path.isObjectProperty()) {
        functionDeclarationStart = node.start || path.get('value.body').node.start;
    } else {
        functionDeclarationStart = node.start;
    }

    var foundedComment = void 0;

    for (var index = comments.length - 1, comment; index >= 0; index--) {
        comment = comments[index];

        if (comment.type === ALLOWED_COMMENT_TYPE && comment.start > previousNodeEnd && comment.end < functionDeclarationStart) {
            foundedComment = comment;
            break;
        }
    }

    if (foundedComment) {
        var directive = getDirective(state.opts);

        if (typeof directive === 'string' && !hasGlobalDirective) {
            foundedComment = foundedComment.value.includes('@' + directive) ? foundedComment : null;
        }

        if (foundedComment) {
            var _comment = foundedComment.value;

            for (var _index = 0, count = ALLOWED_DIRECTIVES.length; _index < count; _index++) {
                if (_comment.includes(ALLOWED_DIRECTIVES[_index])) {
                    return _comment;
                }
            }
        }
    }

    return null;
};